Package: future.mirai
Version: 1.0.0
Depends: future (>= 1.49.0)
Imports: mirai (>= 2.2.0), parallelly, utils
Suggests: future.tests, future.apply, listenv
Title: A 'Future' API for Parallel Processing using 'mirai'
Description: Implementation of the 'Future' API <doi:10.32614/RJ-2021-048> on top of the 'mirai' package <doi:10.5281/zenodo.7912722>. By using this package, you get to take advantage of the benefits of 'mirai' plus everything else that 'future' and the 'Futureverse' adds on top of it. It allows you to process futures, as defined by the 'future' package, in parallel out of the box, on your local machine or across remote machines. Contrary to back-ends relying on the 'parallel' package (e.g. 'multisession') and socket connections, 'mirai_cluster' and 'mirai_multisession', provided here, can run more than 125 parallel R processes. As a reminder, regardless which future backend is used by the user, the code does not have to change, it gives identical results, and behaves exactly the same.
Authors@R: c(
  person("Henrik", "Bengtsson",
         role = c("aut", "cre", "cph"),
         email = "henrikb@braju.com",
         comment = c(ORCID = "0000-0002-7579-5165")
        ),
  person("Charlie", "Gao",
         role = c("ctb"),
         email = "charlie.gao@shikokuchuo.net",
         comment = c(ORCID = "0000-0002-0750-061X", note = "For 'mirai'-related patches and implementing feature requests in 'mirai'")
        ))
License: Apache License (>= 2)
Language: en-US
Encoding: UTF-8
URL: https://future.mirai.futureverse.org,
        https://github.com/futureverse/future.mirai
BugReports: https://github.com/futureverse/future.mirai/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-22 08:18:17 UTC; hb
Author: Henrik Bengtsson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7579-5165>),
  Charlie Gao [ctb] (ORCID: <https://orcid.org/0000-0002-0750-061X>,
    note: For 'mirai'-related patches and implementing feature requests
    in 'mirai')
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2026-06-22 08:40:02 UTC
Built: R 4.6.0; ; 2026-06-22 11:35:20 UTC; unix
