# Generated by roxygen2: do not edit by hand

S3method(getFutureBackendConfigs,MiraiMultisessionFuture)
S3method(interruptFuture,MiraiFutureBackend)
S3method(launchFuture,MiraiFutureBackend)
S3method(nbrOfFreeWorkers,MiraiFutureBackend)
S3method(nbrOfFreeWorkers,MiraiMultisessionFutureBackend)
S3method(nbrOfWorkers,MiraiFutureBackend)
S3method(nbrOfWorkers,MiraiMultisessionFutureBackend)
S3method(resolved,MiraiFuture)
S3method(resolved,MiraiMultisessionFuture)
S3method(result,MiraiFuture)
S3method(result,MiraiMultisessionFuture)
S3method(stopWorkers,MiraiFutureBackend)
S3method(tweak,mirai_cluster)
export(MiraiFutureBackend)
export(MiraiMultisessionFutureBackend)
export(mirai_cluster)
export(mirai_multisession)
importFrom(future,FutureBackend)
importFrom(future,FutureError)
importFrom(future,FutureInterruptError)
importFrom(future,FutureWarning)
importFrom(future,SequentialFutureBackend)
importFrom(future,cancel)
importFrom(future,future)
importFrom(future,interruptFuture)
importFrom(future,launchFuture)
importFrom(future,nbrOfFreeWorkers)
importFrom(future,nbrOfWorkers)
importFrom(future,resolved)
importFrom(future,result)
importFrom(future,run)
importFrom(future,stopWorkers)
importFrom(future,tweak)
importFrom(mirai,call_mirai)
importFrom(mirai,daemons)
importFrom(mirai,is_error_value)
importFrom(mirai,mirai)
importFrom(mirai,status)
importFrom(mirai,stop_mirai)
importFrom(mirai,unresolved)
importFrom(parallelly,availableCores)
importFrom(utils,capture.output)
importFrom(utils,packageVersion)
importFrom(utils,str)
