## Private future.mirai functions
mdebugf <- future.mirai:::mdebugf
mprint <- future.mirai:::mprint
commaq <- future.mirai:::commaq
mirai_daemons_nworkers <- future.mirai:::mirai_daemons_nworkers
mirai_version <- future.mirai:::mirai_version
get_mirai_daemons <- future.mirai:::get_mirai_daemons


